#!/bin/bash
set -euox pipefail

unset GREP_OPTIONS

compatbranches=$(HGRCPATH= hg branches --quiet | grep -E '^mercurial-' | sort -n --reverse)
prev='stable'
topic=${1:-'test-compat'}
for branch in $compatbranches; do
    # Logic in the revsets below:
    # 1. There is target topic on stable or compatibility branch: merge all
    # those commits, they are related to our work and need to pass tests on all
    # compatibility branches.
    # 2. There is no target topic: avoid any commits that have topic set, they
    # are definitely unrelated to what we're doing.
    # In other words, if you want to test certain commits, assign them all to
    # one topic and provide that topic as the first argument to this script.
    uptarget="first(max(branch('re:^$branch') and topic('$topic')) or max(branch('re:^$branch') and not topic()))"
    hg up -r "$uptarget"
    hg topic "$topic"
    mergetarget="first(max(branch('re:^$prev') and topic('$topic')) or max(branch('re:^$prev') and not topic()))"
    if hg log -r "$mergetarget and ancestors(.)" -T 'already merged' | grep 'merge'; then
        echo "not merging $prev into $branch as it is already merged"
    else
        hg merge -r "$mergetarget"
        hg commit -m "test-compat: merge $prev into $branch"
    fi
    prev=$branch
done
